/*
Phi-1 shield and phi-menu for Arduino
Test program: Phi-menu release for Phi-1 and 2 shields 20110312

Programmed by Dr. John Liu
Revision: 03/12/2011
Commercial use without authorization is prohibited.
Find details of the Phi-1 shield, Phi-2 shield or contact Dr. Liu at
http://liudr.wordpress.com/phi-1-shield/
http://liudr.wordpress.com/phi-2-shield/
All rights reserved.

Summary:
This is part of the 20110312 release of phi-menu for phi-1 shields. Please refer to the ducomentation phi-menu_20110312.pdf at http://liudr.wordpress.com/phi-menu.

*/

//Utility functions
void render_multi_line (int line_number)
{
  
}
void render_number_in_place(int number)
{
  char msg[11];
  sprintf(msg,"%d",number);
  lcd.print(msg);
}

void render_00number_in_place(int number)
{
  char msg[11];
  sprintf(msg,"%02d",number);
  lcd.print(msg);
}

void render_char_in_place(int number)
{
  lcd.write(number);
}

void render_YN_in_place(int number)
{
  if (number==0)
  {
    lcd.print("NO");
  }
  else
  {
    lcd.print("YES");
  }
}

//Display strings stored in PROGMEM. Provide the string name stored in PROGMEM to be displayed on the LCD's current cursor position.
void msg_lcd(char* msg_line)
{
  char msg_buffer[17];
  strcpy_P(msg_buffer,msg_line); 
  lcd.print(msg_buffer);
}

//Renders bar graphs on 16*2 displays. Two bars each 10 blocks long will be drawn. max_v-1 gets all the bars. Number of blocks is value/max_v*20.
void render_double_bar(int value, int max_v)
{
  char msg[7];
  int bar1, bar2, extra;
  byte i,j;
  long total_bar;
  total_bar=(20*((long)(value%max_v)))/((long)max_v);
  bar1=total_bar>=10?10:total_bar;
  bar2=total_bar>=10?(total_bar%10):0;
  extra=value/max_v;
  lcd.setCursor(0,0);
  for(i=0;i<bar1;i++)
  {
    lcd.write('\xff');
  }
  for(j=i;j<10;j++)
  {
    lcd.write(' ');
  }
  
  lcd.setCursor(0,1);
  for(i=0;i<bar2;i++)
  {
    lcd.write('\xff');
  }
  for(j=i;j<10;j++)
  {
    lcd.write(' ');
  }
}

int wait_on_escape(int ref_time) // Returns key pressed or 0 if time expires before any key was pressed.
{
//Wait on button push.
  long temp0;
  byte temp1, temp2, temp3, temp4, temp5, temp6;
  temp0=millis();
  do
  {
    temp1=btn_1.sense();
    temp2=btn_2.sense();
    temp3=btn_3.sense();
    temp4=btn_4.sense();
    temp5=btn_5.sense();
    temp6=btn_6.sense();

    if((temp1==buttons_released)||(temp1==buttons_held))
    {
      return(1);
    }
    if((temp2==buttons_released)||(temp2==buttons_held))
    {
      return(2);
    }
    if((temp3==buttons_released)||(temp3==buttons_held))
    {
      return(3);
    }
    if ((temp4==buttons_released)||(temp4==buttons_held))
    {
      return(4);
    }
    if((temp5==buttons_released)||(temp5==buttons_held))
    {
      return(5);
    }
    if((temp6==buttons_released)||(temp6==buttons_held))
    {
      return(6);
    }
  }   while ((millis()-temp0<ref_time));

  return (0);
}


int hmi_with_update_3(int* current, int lower, int upper, int inc, byte column, byte row, byte space, void (*update_function)(int))
/*
HMI with one update function and value wrap-around capability.
This function prints the initial value first so the caller doesn't need to.
Function traps until the update is finalized by the left, right, enter button or escape button.
Return values are updated throught the pointer.
Returns buttons pushed so the caller can determine what to do:
Go back to the last slot with left (-3)
Go forward to the next slot with right (-4)
Enter(1)
Escape(-1).
*/
{
  int number=*current;
  byte temp1;
  char msg[space+1];
  for (byte i=0;i<space;i++) msg[i]=' '; // Create mask the size of the output space.
  msg[space]=0;
  lcd.setCursor(column,row); // Write mask to erase previous content.
  lcd.print(msg);
  lcd.setCursor(column,row); // Write initial value.
  (*update_function)(number);
  lcd.setCursor(column,row); // Write initial value.
  lcd.cursor();

  while(true)
  {
    temp1=wait_on_escape(50);
    switch (temp1)
    {
      case 1:
      if (number+inc<=upper) number+=inc;
      else number=lower;
      lcd.setCursor(column,row);
      lcd.print(msg);
      lcd.setCursor(column,row);
      (*update_function)(number);
      lcd.setCursor(column,row);
      break;
      
      case 2:
      if (number-inc>=lower) number-=inc;
      else number=upper;
      lcd.setCursor(column,row);
      lcd.print(msg);
      lcd.setCursor(column,row);
      (*update_function)(number);
      lcd.setCursor(column,row);
      break;
      
      case 3: // Left is pressed
      *current=number;
      lcd.noCursor();
      return (-3);
      break;
      
      case 4: // Right is pressed
      *current=number;
      lcd.noCursor();
      return (-4);
      break;
      
      case 5: // Enter is pressed
      *current=number;
      lcd.noCursor();
      return(1);
      break;
      
      case 6: // Escape is pressed
      lcd.noCursor();
      return (-1);
      break;
      
      default:
      break;
    }
  }
}

int input_panel3(char* msg, byte length, byte column, byte row, char _start, char _end)
/*
Alphanumerical input panel for texts up to 16 characters.
The option starts with the character _start and ends with character _end
The total length will not exceed length.
The first letter appears at column, row
The function prints the content of the buffer msg before polling for input
The function returns number of actual characters
The function returns -1 if the input is cancelled.
The function fills the buffer after the last character with \0 only if the buffer is not filled. The caller is responsible to fill the character beyond the end of the buffer with \0.
*/
{
  byte pointer=0;
  int temp1, temp2;
  lcd.setCursor(column,row);
  lcd.print(msg);
  while (pointer<length)
  {
    temp1=0;
    temp2=msg[pointer];
    lcd.setCursor(pointer,row);
    temp1=hmi_with_update_3(&temp2, _start, _end, 1, column+pointer, row, 1, render_char_in_place); // RTC library already adds 2000 to returned value in rtc[6].
    switch (temp1)
    {
      case -1: // Escape
      return (-1);
      break;
      
      case -3: // Left
      msg[pointer]=lowByte(temp2);
      if (pointer>0) pointer--;
      else return(-1); //When you are on the left most spot, pressing left one more time equals escape. Comment this line to disable this function.
      break;
      
      case -4: // Right.
      msg[pointer]=lowByte(temp2);
      if (pointer<length-1) pointer++;
      else return (pointer+1); //When you are on the right most spot, pressing right one more time equals confirm. Comment this line to disable this function.
      break;
      
      case 1: // Enter
      msg[pointer]=lowByte(temp2);
//      if (pointer+1<length) msg[pointer+1]=0;
      lcd.noCursor();
      return(pointer+1);
      break;
      
      default:
      break;
    }
  }
}
  
/*
void render_RTC(int temp)
{
  int rtc[7];
  char msg[17];
  RTC.get(rtc,true);
  lcd.clear();
  switch(temp)
  {
    case 0:
    render_month_in_place(rtc[5]-1);//  render_00number_in_place(rtc[5]);
    sprintf(msg,"/%02d/%4d ",rtc[4],rtc[6]);//  lcd.print("JAN/01/2000 SUN");
    lcd.print(msg);
    render_DOW_in_place(rtc[3]-1);//  render_00number_in_place(rtc[3]);
    lcd.setCursor(4,1);
    sprintf(msg,"%2d:%02d:%02d",rtc[2],rtc[1],rtc[0]);
    lcd.print(msg);
    break;
    
    case 1:
    lcd.setCursor(4,1);
    sprintf(msg,"%2d:%02d:%02d",rtc[2],rtc[1],rtc[0]);
    lcd.print(msg);
    break;
    
    default:
    break;
  }
}

void render_DOW_in_place(int number)
{
  msg_lcd(dow_00+(number%8)*4); // *4 because DOW is 3-character+ zero
}

void render_alarm_in_place(int number)
{
  msg_lcd(alarm_00+(number%5)*8); // *4 because alarm is 7-character+ zero
}

void render_month_in_place(int number)
{
  msg_lcd(month_00+(number%12)*4); // *4 because month is 3-character+ zero
}
*/

void render_phi_menu_style(int number)
{
//  msg_lcd((char*)*(phi_menu_styles[number])); // Three styles right now
  char menu_buffer[20];
  strcpy_P(menu_buffer,(char*)pgm_read_word(&(phi_menu_styles[number]))); 
  lcd.print(menu_buffer);
}
//%(sizeof(phi_menu_styles)/sizeof(char*)))
