/*
Phi-1 shield and phi-menu for Arduino
Test program: Phi-menu release for Phi-1 and 2 shields 20110312

Programmed by Dr. John Liu
Revision: 03/12/2011
Commercial use without authorization is prohibited.
Find details of the Phi-1 shield, Phi-2 shield or contact Dr. Liu at
http://liudr.wordpress.com/phi-1-shield/
http://liudr.wordpress.com/phi-2-shield/
All rights reserved.

Summary:
This is part of the 20110312 release of phi-menu for phi-1 shields. Please refer to the ducomentation phi-menu_20110312.pdf at http://liudr.wordpress.com/phi-menu.

*/

// Menu texts
PROGMEM prog_char menu_item00[]="Item #1";
PROGMEM prog_char menu_item01[]="Item #2";
PROGMEM prog_char menu_item02[]="Item #3";
PROGMEM prog_char menu_item03[]="Introduction";
PROGMEM prog_char menu_item04[]="Settings";
PROGMEM prog_char menu_item05[]="Credits";

PROGMEM const char *menu_item[] = {menu_item00, menu_item01, menu_item02, menu_item03, menu_item04, menu_item05};

//The following are related to the menu
int menu_pointer=0;
int phi_menu_style=phi_menu_style_classic;// Choose among phi_menu_style_classic, phi_menu_style_2X2, and phi_menu_style_index

//These are related to the specific program
int _delay=2000;
//This program is the main menu. It handles inputs from the keys, updates the menu or executes a certain menu function accordingly.
void do_menu()
{
  //Do not change this
  int btn_pressed;
  int menu_buffer=menu_pointer;
  switch(phi_menu_style)  
  {
    case phi_menu_style_classic:
    btn_pressed=hmi_with_update_3(&menu_pointer,0,sizeof(menu_item)/sizeof(char*)-1,1,16,1,0,render_menu_classic); // Sticky menu item.
    break;
    
    case phi_menu_style_2X2:
    btn_pressed=hmi_with_update_3(&menu_pointer,0,sizeof(menu_item)/sizeof(char*)-1,1,16,1,0,render_menu_2X2); // Sticky menu item.
    break;
    
    case phi_menu_style_index:
    btn_pressed=hmi_with_update_3(&menu_pointer,0,sizeof(menu_item)/sizeof(char*)-1,1,16,1,0,render_menu_index); // Sticky menu item.
    break;
    
    default:
    break;
  }

  //Menu will treat keys such as left, right, and confirm all as confirm, for maximal compatibility.
  menu_pointer=(btn_pressed==-1)?menu_buffer:menu_pointer; // In case escape was triggered, value doesn't change.
  switch (menu_pointer)
  {//You may change the content inside this switch. Replace with your menu items you defined in the defs.h and the functions they run, such as _display() and _parameter().
    case 0:
    _menu_function_0();
    break;

    case 1:
    _menu_function_1();
    break;
    
    case 2:
    _menu_function_2();
    break;
    
    case 3:
    _menu_function_3();
    break;

    case 4:
    _menu_function_4();
    break;
    
    case 5:
    _menu_function_5();
    break;
    
    case 6:
    _menu_function_6();
    break;

    case 7:
    _menu_function_7();
    break;
    
    case 8:
    _menu_function_8();
    break;
    
    case 9:
    _menu_function_9();
    break;

    default:
    break;
  }
}

// Menu functions go here. The functions are called when their menu items are selected. They are called only once so if you want to do something repeatedly, make sure you have a while loop.
void _menu_function_0() //Asks you for password for a given number of trials
{
  lcd.clear();
  lcd.print("This is a demo!");
  wait_on_escape(_delay);
}

void _menu_function_1() //Shows credit once more
{
  lcd.clear();
  lcd.print("Just another");
  lcd.setCursor(0,1);
  lcd.print("demo!");
  wait_on_escape(_delay);
}

void _menu_function_2() //Fill this with your code or leave it empty
{
  lcd.clear();
  lcd.print("One last demo!");
  wait_on_escape(_delay);
}

void _menu_function_3() //Fill this with your code or leave it empty
{
  lcd.clear();
  lcd.print("Phi-menu distrib");
  lcd.setCursor(0,1);
  lcd.print("-ution 20110312");
  wait_on_escape(_delay);
  lcd.clear();
  lcd.print("for Phi-1 shield");
  lcd.setCursor(0,1);
  lcd.print("3/12/2011 Dr.Liu");
  wait_on_escape(_delay);
  lcd.clear();
}

void _menu_function_4() //Settings
{
  int para_val, btn_pressed;

  lcd.clear(); // Input
  lcd.print("Menu style");
  para_val=phi_menu_style;
  btn_pressed=hmi_with_update_3(&para_val, 0, 2, 1, 0, 1, 16, render_phi_menu_style); // Asks number of trials
  phi_menu_style=(btn_pressed==-1)?phi_menu_style:para_val; // In case of escape, value doesn't change.

  delay(100);
  lcd.clear(); // Input
  lcd.print("Delay(ms):");
  para_val=_delay;
  btn_pressed=hmi_with_update_3(&para_val, 1000, 4000, 500, 4, 1, 4, render_number_in_place); // Asks number of trials
  _delay=(btn_pressed==-1)?_delay:para_val; // In case of escape, value doesn't change.
}

void _menu_function_5() //Fill this with your code or leave it empty
{
  show_credit();
}

void _menu_function_6() //Fill this with your code or leave it empty
{
}

void _menu_function_7() //Fill this with your code or leave it empty
{
}

void _menu_function_8() //Fill this with your code or leave it empty
{
}

void _menu_function_9() //Fill this with your code or leave it empty
{
}

//Display the LCD menu according to argument me.
void render_menu_classic(int me)
{
  char menu_buffer[20];
  lcd.clear();
  msg_lcd(msg_07);
  lcd.setCursor(0,1);
  strcpy_P(menu_buffer,(char*)pgm_read_word(&(menu_item[me]))); 
  lcd.print(menu_buffer);
}

void render_menu_2X2(int me)
{
  byte _first_item, _last_item; // Which items to display.
  char menu_buffer[20];
  lcd.clear();
  _first_item=(me/4)*4;
  _last_item=_first_item+3;
  if (_last_item>sizeof(menu_item)/sizeof(char*)-1) _last_item=sizeof(menu_item)/sizeof(char*)-1;
  for (byte i=_first_item;i<=_last_item;i++)
  {
    strcpy_P(menu_buffer+1,(char*)pgm_read_word(&(menu_item[i]))); //menu_buffer+1 makes space for space or great than symbol.
    menu_buffer[8]=0;
//    menu_buffer[0]=(i==me)?'>':'1'+i%10;// Show ">" or menu item number mod 10
    menu_buffer[0]=(i==me)?'\x7e':'\xA5';// Show ">" or a dot
    lcd.setCursor(8*((i-_first_item)%2),(i-_first_item)/2);
//    Serial.println(8*((i-_first_item)%2)*10+(i-_first_item)/2);
    lcd.print(menu_buffer);
  }
}

//#define underscore_indicator
#define symbol_indicator
void render_menu_index(int me) // This menu is very similar to the classic menu except it also shows a numerical index of the menu items on top line. The active item number is marked such as "<".
{
  char menu_buffer[20];
  lcd.clear();
  strcpy_P(menu_buffer,(char*)pgm_read_word(&(menu_item[me]))); 
  lcd.print(menu_buffer);
  lcd.setCursor(0,1);

#ifdef symbol_indicator
  for (byte i=0;i<sizeof(menu_item)/sizeof(char*);i++)
  {
    if (i==me) lcd.write('^');
    else lcd.write(i%10+'1');
  }

#endif

#ifdef underscore_indicator // Not working yet since hmi_with_update controls where the cursor appears.
  for (byte i=0;i<sizeof(menu_item)/sizeof(char*);i++)
  {
    lcd.write(i+'1');
  }
  lcd.setCursor(me,1);
  delay(100);
#endif
}
