/*
Phi-1 shield and phi-menu for Arduino
Test program: Phi-menu release for Phi-1 and 2 shields 20110312
To choose between phi-1 and phi-2 shields, uncomment line 20 or 21, the corresponding #define such as #define phi_1_shield.
For phi-2 shield, connect the buzzer to analog 2.

Programmed by Dr. John Liu
Revision: 03/12/2011
Commercial use without authorization is prohibited.
Find details of the Phi-1 shield, Phi-2 shield or contact Dr. Liu at
http://liudr.wordpress.com/phi-1-shield/
http://liudr.wordpress.com/phi-2-shield/
All rights reserved.

Summary:
This is part of the 20110312 release of phi-menu for phi-1 shields. Please refer to the ducomentation phi-menu_20110312.pdf at http://liudr.wordpress.com/phi-menu.

*/

#define phi_1_shield
//#define phi_2_shield


#include <LiquidCrystal.h>
#include <WProgram.h>
#include <Wire.h>
#include <stdio.h>
#include <avr/pgmspace.h>
#include <EEPROM.h>
#include "defs.h"
#include "progmem.h"
#include "phi_buttons.h"

LiquidCrystal lcd(LCD_RS,LCD_EN,LCD_D4,LCD_D5,LCD_D6,LCD_D7); // Create the lcd object

/*
These are initializations of the buttons.
There are many scenarios you can use. A user can operate the menu and input with as many as 6 buttons for convenience or as few as one button to save Arduino I/O pins. Just enable the respective define. Right now the 6-button scenario is used.
use_6_btn: all 6 buttons are defined for maximal convenience to the operator. Operator may jog in text input.
use_5_btn: all but the “left” are defined for convenience to the operator but there is no way to jog on text input.
use_4_btn: all but “left” and “right” are defined for medium convenience to the operator. This works best when all you do is entering numbers of choosing from a list and don't use text input.
use_3_btn: only “up”, “right” and “confirm” are defined to save Arduino pins offering less convenience to the operator. This works alright if input options are limited such as month or password numbers.
use_2_btn: only “up” and “right” are defined to save Arduino pins. The operator will still be able to operate everything but entering text will be a pain.
use_1_btn: This enables auto button on “up” and only defines “right”. This way saves most Arduino pins since only one button is used but the operator is only expected to enter limited information.
*/
#define use_6_btn
//#define use_5_btn
//#define use_4_btn
//#define use_3_btn
//#define use_2_btn
//#define use_2_btn
//#define use_1_btn

#ifdef use_1_btn
phi_buttons btn_1(btn_auto, LOW);
phi_buttons btn_2(btn_null, LOW);
phi_buttons btn_3(btn_null, LOW);
phi_buttons btn_4(btn_r, LOW);
phi_buttons btn_5(btn_null, LOW);
phi_buttons btn_6(btn_null, LOW);
#endif

#ifdef use_2_btn
phi_buttons btn_1(btn_u, LOW);
phi_buttons btn_2(btn_null, LOW);
phi_buttons btn_3(btn_null, LOW);
phi_buttons btn_4(btn_r, LOW);
phi_buttons btn_5(btn_null, LOW);
phi_buttons btn_6(btn_null, LOW);
#endif

#ifdef use_3_btn
phi_buttons btn_1(btn_u, LOW);
phi_buttons btn_2(btn_null, LOW);
phi_buttons btn_3(btn_null, LOW);
phi_buttons btn_4(btn_r, LOW);
phi_buttons btn_5(btn_b, LOW);
phi_buttons btn_6(btn_null, LOW);
#endif

#ifdef use_4_btn
phi_buttons btn_1(btn_u, LOW);
phi_buttons btn_2(btn_d, LOW);
phi_buttons btn_3(btn_null, LOW);
phi_buttons btn_4(btn_null, LOW);
phi_buttons btn_5(btn_b, LOW);
phi_buttons btn_6(btn_a, LOW);
#endif

#ifdef use_5_btn
phi_buttons btn_1(btn_u, LOW);
phi_buttons btn_2(btn_d, LOW);
phi_buttons btn_3(btn_null, LOW);
phi_buttons btn_4(btn_r, LOW);
phi_buttons btn_5(btn_b, LOW);
phi_buttons btn_6(btn_a, LOW);
#endif

#ifdef use_6_btn
phi_buttons btn_1(btn_u, LOW);
phi_buttons btn_2(btn_d, LOW);
phi_buttons btn_3(btn_l, LOW);
phi_buttons btn_4(btn_r, LOW);
phi_buttons btn_5(btn_b, LOW);
phi_buttons btn_6(btn_a, LOW);
#endif

void setup()
{
  byte ch_buffer[10]; // This buffer is required for custom characters on the LCD.
  lcd.begin(16, 2);
//  Serial.begin(115200);

//Programming custom characters. To enable this feature, go to progmem.h and uncomment the prog_char definitions. The custom characters will be lost when power is cut.
#ifdef prog_char_now
  for (int i=0;i<8;i++)
  {
    strcpy_P((char*)ch_buffer,(char*)pgm_read_word(&(ch_item[i])));
    lcd.createChar(i, ch_buffer);
  }
#endif

// Set the two channels to output to drive the buzzer and LED.
  pinMode(buzzer,OUTPUT);
  digitalWrite(buzzer,LOW);
  pinMode(led,OUTPUT);
  digitalWrite(led,LOW);
  show_credit();
}

void loop()
{
  do_menu();
}

void show_credit()
{
  // Display credits
  lcd.clear();
  lcd.print("Phi-menu release");
  lcd.setCursor(0,1);
  lcd.print("Dr.Liu 03/12/11");
  wait_on_escape(2000);
  lcd.clear();
  lcd.print("http://liudr.");
  lcd.setCursor(0,1);
  lcd.print("wordpress.com");
  wait_on_escape(2000);
  lcd.clear();
}
